#include <avr/io.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <math.h>
#include <avr/interrupt.h>
#include "utility.h"
#include "init.h"	

#define AMBIENT_LIGHT 0

unsigned int IRtoDist(unsigned int IR);
unsigned char DisttoServo(unsigned char distance);

volatile bool readIRavg = false, readIR = false, setServoFlag = true, downCounting = false;
volatile unsigned int IRdist = 0; 
volatile unsigned int IRacc = 0, servoAvg[10];
volatile unsigned char IRcounter = 0;


int main(void)
{	

	cli();
	init_ports();
	init_signal();		
	init_timers();
	init_ADC();
	sei();
	setIR(8);

	while(1)
	{
		if (setServoFlag)
		{
			unsigned int servo = 0;
			for(unsigned char cnt=0; cnt<9; cnt++)
				servoAvg[cnt] = servoAvg[cnt+1];

			if(AMBIENT_LIGHT)
			{
				servoAvg[9] = (unsigned char)(IRacc>>5)*2;
				IRacc = 0;
				IRcounter = 0;
			}
			else
			{
				servoAvg[9] = DisttoServo(IRtoDist(IRdist));
//				setServo(servoAvg[9]);
			}
			for(unsigned char cnt=0; cnt<10;cnt++)
				servo += servoAvg[cnt];
			servo = servo/10;
			setServo((unsigned char)(servo));
			setServoFlag = false;
		}
	}

	return 0;

}


unsigned int IRtoDist(unsigned int IR)
{
	double distance = 103.0*pow((1.0/((double)IR)),0.52);
	return (unsigned int)distance;
}

unsigned char DisttoServo(unsigned char distance)
{
	unsigned char servo = (distance*1.5)+6;
	return servo;
}

ISR(ADC_vect)
{
	unsigned int low = 0, high = 0; 
	low = ADCL; 
	high = ADCH;	

	if(readIRavg)
	{
		if(IRcounter < 32)
		{
			ADCSRA |= 0b01000000; // Start ADC		
			IRacc += (high<<8)+low; 
			IRcounter++;
		}
		else
		{
			readIRavg = false;
			setServoFlag = true;
		}
	}
	else if(readIR)
	{
		if(AMBIENT_LIGHT)
		{
			IRdist = ((high<<8)+low);
			readIR = false;
		}
		else
		{
			IRdist = ((high<<8)+low) - (IRacc>>5);
			IRacc = 0;
			IRcounter = 0;
			setServoFlag = true;
			readIR = false;
		}
	}

}



SIGNAL(TIM0_OVF_vect) 
{	

	readIRavg = true;
	ADCSRA |= 0b01000000; // Start ADC
}

SIGNAL(TIM0_COMPB_vect)
{
	if(downCounting)
	{
		downCounting = false; 
	}
	else
	{
		readIR = true;
		ADCSRA |= 0b01000000; // Start ADC
		downCounting = true;
	}
}

